#!/usr/bin/env python
import gi
from gi.repository import TrackerMiner, GLib, GObject, Gio


class MyMiner (TrackerMiner.Miner):
    __gtype_name__ = 'MyMiner'

    def __init__ (self):
        TrackerMiner.Miner.__init__ (self,
                                     name="MyMiner",
                                     progress=0,
                                     status="fine")
        # This shouldn't be needed, but at the moment the
        # overrided methods are not called
        self.connect ("started", self.started_cb)

        # Say to initable that we are ok
        self.init (None)

    def started (self, x):
        print "override started"

    def started_cb (self, x):
        print "started as callback"

    def stopped (self):
        print "override stopped"

    def resumed (self):
        print "override resumed"

    def paused (self):
        print "override paused"

    def progress (self):
        print "override progress"

if __name__ == "__main__":
    m = MyMiner ()
    m.start ()

    GObject.MainLoop().run ()



#!/usr/bin/env python
import gi
from gi.repository import Tracker, GObject


def results_ready_cb (obj, result, user_data):
    cursor = obj.query_finish (result)

    # This can also be done asynchronously
    while (cursor.next (None)):
        print cursor.get_string (0)

    user_data.quit ()

def connection_ready_cb (object, result, user_data):
    assert user_data
    conn = Tracker.SparqlConnection.get_finish (result)

    conn.query_async ("SELECT ?u WHERE { ?u a nie:InformationElement. }",
                      None,
                      results_ready_cb,
                      user_data)


if __name__ == "__main__":
    loop = GObject.MainLoop ()

    Tracker.SparqlConnection.get_async (None, connection_ready_cb, loop)

    loop.run ()



#!/usr/bin/env python
import gi
from gi.repository import Tracker, GObject

def results_ready_cb (obj, result, user_data):
    cursor = obj.query_finish (result)

    # This can also be done asynchronously
    while (cursor.next (None)):
        print cursor.get_string (0)

    user_data.quit ()


if __name__ == "__main__":
    loop = GObject.MainLoop ()

    # The connection can be requested asynchronously
    conn = Tracker.SparqlConnection.get (None)
    conn.query_async ("SELECT nie:url(?u) WHERE { ?u a nfo:FileDataObject }",
                      None,
                      results_ready_cb,
                      loop)

    loop.run ()



#!/usr/bin/python3
#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
# Copyright (C) 2019, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
"""
Send concurrent inserts and queries to the daemon to check the concurrency.
"""
from gi.repository import GLib

import unittest as ut
from storetest import CommonTrackerStoreTest as CommonTrackerStoreTest

AMOUNT_OF_TEST_INSTANCES = 100
AMOUNT_OF_QUERIES = 10


class TestConcurrentQuery (CommonTrackerStoreTest):
    """
    Send a bunch of queries to the daemon asynchronously, to test the queue
    holding those queries
    """

    def setUp(self):
        self.main_loop = GLib.MainLoop()

        self.mock_data_insert()
        self.finish_counter = 0

    def mock_data_insert(self):
        query = "INSERT {\n"
        for i in range(0, AMOUNT_OF_TEST_INSTANCES):
            query += "<test-09:instance-%d> a nco:PersonContact ; nco:fullname 'moe %d'.\n" % (
                i, i)
        query += "}"
        self.tracker.update(query)

    def mock_data_delete(self):
        query = "DELETE {\n"
        for i in range(0, AMOUNT_OF_TEST_INSTANCES):
            query += "<test-09:instance-%d> a rdfs:Resource.\n" % (i)
        query += "}"
        self.tracker.update(query)

        query = "DELETE {\n"
        for i in range(0, AMOUNT_OF_QUERIES):
            query += "<test-09:picture-%d> a rdfs:Resource.\n" % (i)
        query += "}"
        self.tracker.update(query)

    def test_async_queries(self):
        QUERY = "SELECT ?u WHERE { ?u a nco:PersonContact. FILTER regex (?u, 'test-09:ins')}"
        UPDATE = "INSERT { <test-09:picture-%d> a nmm:Photo. }"
        for i in range(0, AMOUNT_OF_QUERIES):
            self.tracker.query(
                QUERY,
                result_handler=self.reply_cb,
                error_handler=self.error_handler)
            self.tracker.update(
                UPDATE % (i),
                result_handler=self.update_cb,
                error_handler=self.error_handler)

        # Safeguard of 60 seconds. The last reply should quit the loop
        GLib.timeout_add_seconds(60, self.timeout_cb)
        self.main_loop.run()

    def reply_cb(self, obj, results, data):
        self.finish_counter += 1
        self.assertEqual(len(results), AMOUNT_OF_TEST_INSTANCES)
        if (self.finish_counter >= AMOUNT_OF_QUERIES):
            self.timeout_cb()

    def update_cb(self, obj, results, data):
        self.assertTrue(True)

    def error_handler(self, obj, e, user_data):
        print("ERROR in DBus call: %s" % e)
        raise(e)

    def timeout_cb(self):
        self.mock_data_delete()
        self.main_loop.quit()
        return False

if __name__ == "__main__":
    ut.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
# Copyright (C) 2019, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

"""
Stand-alone tests cases for the store, checking the collation is working
"""
import time
import random
import locale

import unittest as ut
from storetest import CommonTrackerStoreTest as CommonTrackerStoreTest


class TrackerStoreCollationTests (CommonTrackerStoreTest):
    """
    Insert few instances with a text field containing collation-problematic words.
    Ask for those instances order by the field and check the results.
    """

    def setUp(self):
        """
        Each test append to this list the used URIS, so they can be removed
        in the tearDown
        """
        self.clean_up_instances = []

    def tearDown(self):
        for uri in self.clean_up_instances:
            self.tracker.update("DELETE { <%s> a rdfs:Resource. }" % (uri))
        self.clean_up_instances = []
        time.sleep(1)

    def __insert_text(self, text):
        uri = "test://collation-01-%d" % (random.randint(1, 1000))
        # There is a remote chance to get a duplicate int
        while (uri in self.clean_up_instances):
            uri = "test://collation-01-%d" % (random.randint(1, 1000))
        self.clean_up_instances.append(uri)

        self.tracker.update ("""
        INSERT {
            <%s> a nie:InformationElement ;
                nie:title "%s" ;
                nie:description "tracker-collation-test-instance" .
        }
         """ % (uri, text))

    def get_text_sorted_by_collation(self):
        return self.tracker.query ("""
         SELECT ?title WHERE {
            ?u a nie:InformationElement ;
               nie:title ?title ;
               nie:description 'tracker-collation-test-instance' .
         } ORDER BY ?title
        """)

    def __collation_test(self, input_list, expected_list):

        for i in input_list:
            self.__insert_text(i)

        results = [r[0] for r in self.get_text_sorted_by_collation()]
        self.assertEqual(len(results), len(expected_list))

        for r in range(0, len(results)):
            self.assertEqual(results[r], expected_list[r],
                             """Error:
                                  Expected : *** %s
                                  Result   : *** %s
                                  Using locale (%s, %s)
                               """ % (expected_list,
                                      results,
                                      locale.getdefaultlocale()[0],
                                      locale.getdefaultlocale()[1]))

    def test_collation_01(self):
        """
        Behaves as case-insensitive
        """
        input_dt = ["abb", "bb",  "Abc", "Ba"]
        expected = ["abb", "Abc", "Ba",  "bb"]
        self.__collation_test(input_dt, expected)

    def test_collation_02(self):
        """
        In conflict, Capital letters go *after* small letters
        """
        input_dt = ["Bb", "bb", "aa", "Aa"]
        expected = ["aa", "Aa", "bb", "Bb"]
        self.__collation_test(input_dt, expected)

    def test_collation_03(self):
        """
        Example from the unicode spec
        http://www.unicode.org/reports/tr10/#Main_Algorithm
        """
        input_dt = ["Cab", "cab", "dab", "cáb"]
        expected = ["cab", "Cab", "cáb", "dab"]
        self.__collation_test(input_dt, expected)

    def test_collation_04(self):
        """
        Spanish test in english locale
        """
        input_dt = ["ä", "ö", "a", "e", "i", "o", "u"]
        expected = ["a", "ä", "e", "i", "o", "ö", "u"]
        self.__collation_test(input_dt, expected)

if __name__ == "__main__":
    print("""
    TODO:
      * Check what happens in non-english encoding
      * Dynamic change of collation (not implemented yet in tracker)
    """)
    ut.main()



#!/usr/bin/python3
#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
# Copyright (C) 2019, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

"""
These tests use only the store. They insert instances with known text
and run sparql with fts functions to check the results.
"""
import unittest as ut
from storetest import CommonTrackerStoreTest as CommonTrackerStoreTest


class TestFTSFunctions (CommonTrackerStoreTest):
    """
    Insert data with text and check the fts:xxxx functions are returning the expected results
    """
    @ut.skip("Test currently fails.")
    def test_fts_rank(self):
        """
        1. Insert a Contact1 with 'abcdefxyz' as fullname and nickname
        2. Insert a Contact2 with 'abcdefxyz' as fullname
        2. Insert a Contact3 with 'abcdefxyz' as fullname and twice in nickname
        3. Query sorting by fts:rank
           EXPECTED: The 3 contacts in order: 3, 1, 2
        4. Remove the created resources
        """
        insert_sparql = """
        INSERT {
        <contact://test/fts-function/rank/1> a nco:PersonContact ;
                       nco:fullname 'abcdefxyz' ;
                       nco:nickname 'abcdefxyz' .

        <contact://test/fts-function/rank/2> a nco:PersonContact ;
                       nco:fullname 'abcdefxyz' .

        <contact://test/fts-function/rank/3> a nco:PersonContact ;
                       nco:fullname 'abcdefxyz' ;
                       nco:nickname 'abcdefxyz abcdefxyz' .
        }
        """
        self.tracker.update(insert_sparql)

        query = """
        SELECT ?contact WHERE {
           ?contact a nco:PersonContact ;
                fts:match 'abcdefxyz' .
        } ORDER BY DESC (fts:rank(?contact))
        """
        results = self.tracker.query(query)

        self.assertEqual(len(results), 3)
        self.assertEqual(results[0][0], "contact://test/fts-function/rank/1")
        self.assertEqual(results[1][0], "contact://test/fts-function/rank/2")
        self.assertEqual(results[2][0], "contact://test/fts-function/rank/3")

        delete_sparql = """
        DELETE {
        <contact://test/fts-function/rank/1> a rdfs:Resource .
        <contact://test/fts-function/rank/2> a rdfs:Resource .
        <contact://test/fts-function/rank/3> a rdfs:Resource .
        }
        """
        self.tracker.update(delete_sparql)

    def test_fts_offsets(self):
        """
        1. Insert a Contact1 with 'abcdefxyz' as fullname and nickname
        2. Insert a Contact2 with 'abcdefxyz' as fullname
        2. Insert a Contact3 with 'abcdefxyz' as fullname and twice in nickname
        3. Query fts:offsets for 'abcdefxyz'
           EXPECTED: The 3 contacts in insertion order, with 2, 1 and 3 pairs (prop, offset=1) each
        4. Remove the created resources
        """
        insert_sparql = """
        INSERT {
        <contact://test/fts-function/offset/1> a nco:PersonContact ;
                       nco:fullname 'abcdefxyz' ;
                       nco:nickname 'abcdefxyz' .

        <contact://test/fts-function/offset/2> a nco:PersonContact ;
                       nco:fullname 'abcdefxyz' .

        <contact://test/fts-function/offset/3> a nco:PersonContact ;
                       nco:fullname 'abcdefxyz' ;
                       nco:nickname 'abcdefxyz abcdefxyz' .
        }
        """
        self.tracker.update(insert_sparql)

        query = """
        SELECT fts:offsets (?contact) WHERE {
           ?contact a nco:PersonContact ;
                fts:match 'abcdefxyz' .
        }
        """
        results = self.tracker.query(query)

        self.assertEqual(len(results), 3)
        self.assertEqual(results[0][0], 'nco:fullname,0,nco:nickname,0')
        self.assertEqual(results[1][0], 'nco:fullname,0')
        self.assertEqual(
            results[2][0], 'nco:fullname,0,nco:nickname,0,nco:nickname,10')

        delete_sparql = """
        DELETE {
        <contact://test/fts-function/offset/1> a rdfs:Resource .
        <contact://test/fts-function/offset/2> a rdfs:Resource .
        <contact://test/fts-function/offset/3> a rdfs:Resource .
        }
        """
        self.tracker.update(delete_sparql)


if __name__ == '__main__':
    ut.main()



#!/usr/bin/env python3
#
# Copyright (C) 2012-2013 Martyn Russell <martyn@lanedo.com>
# Copyright (C) 2012      Sam Thursfield <sam.thursfield@codethink.co.uk>
# Copyright (C) 2016      Sam Thursfield <sam@afuera.me.uk>
#
# This script allows a user to utilise Tracker for local instances by
# specifying an index directory location where the Tracker data is
# stored and a content directory location where the content to be
# indexed is kept. From there, queries or a shell can be launched to
# use that data.
#
# This was initially a shell script by Sam and later converted into a
# more comprehensive python script by Martyn.
#
# Usage:
#  - Create or update an index stored in tracker/ subdir with content in html/
#      tracker-sandbox.py -i tracker -c html -u
#  - Query for 'foo'
#      tracker-sandbox.py -i tracker -c html -q foo
#  - List files in index
#      tracker-sandbox.py -i tracker -c html -l
#  - Start shell with environment set up
#      tracker-sandbox.py -i tracker -c html -s
#  - Test with different prefixes, e.g. /usr/local installs
#      tracker-sandbox.py -i tracker -c html -s -p /usr/local
#  ...
#
# Changes:
#  - If you make _ANY_ changes, please send them in so I can incorporate them.
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#

import locale
import os
import subprocess
import optparse
import signal
import sys
import errno

import configparser

import gi
gi.require_version('Tracker', '2.0')
from gi.repository import Tracker, GLib

# Script
script_name = 'tracker-sandbox'
script_version = '0.1'
script_about = 'Localised Tracker sandbox for content indexing and search'

index_location_abs = ''

default_prefix = '/usr'
default_debug_verbosity = 2

# Session
dbus_session_pid = -1
dbus_session_address = ''
dbus_session_file = ''

store_pid = -1
store_proc = None

original_xdg_data_home = GLib.get_user_data_dir()

# Template config file
config_template = """
[General]
verbosity=0
sched-idle=0
initial-sleep=0

[Monitors]
enable-monitors=false

[Indexing]
throttle=0
index-on-battery=true
index-on-battery-first-time=true
index-removable-media=false
index-optical-discs=false
low-disk-space-limit=-1
index-recursive-directories=;
index-single-directories=;
ignored-directories=;
ignored-directories-with-content=;
ignored-files=
crawling-interval=-1
removable-days-threshold=3

[Writeback]
enable-writeback=false
"""

# Utilities


def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST:
            pass
        else:
            raise


def debug(message):
    if opts.debug:
        print(message)

# DB functions (sync for now)


def db_query_have_files():
    # Set this here in case we used 'bus' for an update() before this.
    # os.environ['TRACKER_SPARQL_BACKEND'] = 'direct'

    print('Using query to check index has data in it...')

    conn = Tracker.SparqlConnection.get(None)
    cursor = conn.query(
        'select count(?urn) where { ?urn a nfo:FileDataObject }', None)

    # Only expect one result here...
    while (cursor.next(None)):
        print('  Currently %d file(s) exist in our index' %
              (cursor.get_integer(0)))


def db_query_list_files():
    # Set this here in case we used 'bus' for an update() before this.
    # os.environ['TRACKER_SPARQL_BACKEND'] = 'direct'

    print('Using query to list files indexed...')

    conn = Tracker.SparqlConnection.get(None)
    cursor = conn.query(
        'select nie:url(?urn) where { ?urn a nfo:FileDataObject }', None)

    # Only expect one result here...
    while (cursor.next(None)):
        print('  ' + cursor.get_string(0)[0])


def db_search(search_text):
    conn = Tracker.SparqlConnection.get(None)
    query = ('select nie:url(?urn) where { ?urn a nfo:FileDataObject . '
             '?urn fts:match "%s" }')
    cursor = conn.query(query % (search_text), None)

    print('Found:')

    while (cursor.next(None)):
        print('  ' + cursor.get_string(0)[0])


def db_sparql_query(sparql):
    conn = Tracker.SparqlConnection.get(None)
    cursor = conn.query(sparql)

    print('Results:')

    while (cursor.next(None)):
        row = []
        for column in range(0, cursor.get_n_columns()):
            row.append(cursor.get_string(column)[0])
        print('  ' + '\t'.join(row))


# Index functions

def index_clean():
    # tracker reset --hard
    debug('Cleaning index, FIXME: Does nothing.')


def find_libexec_binaries(command):
    binary = os.path.join(opts.prefix, 'libexec', command)
    if not os.path.exists(binary):
        binary = os.path.join(opts.prefix, 'libexec', command)
        if not os.path.exists(binary):
            return None

    return binary


def index_update():
    debug('Updating index ...')
    debug('--')

    # FIXME: Need to start tracker-extract to make sure extended
    # metadata is created, but the problem is, after miner-fs
    # stops, we return to the prompt, so how do we handle that?
    #
    # We need to listen to signals from tracker-extract and then
    # quit after some inactivity I think ... OR listen to
    # GraphUpdated and when there are no more objects without a
    # data-source, we know all data was indexed.

    # Start tracker-miner-fs
    binary = find_libexec_binaries('tracker-miner-fs')
    if binary is None:
        print('Could not find "tracker-miner-fs" in $prefix/lib{exec} '
              'directories', file=sys.stderr)
        print('Is Tracker installed properly?', file=sys.stderr)
        sys.exit(1)

    try:
        # Mine data WITHOUT being a daemon, exit when done. Ignore desktop
        # files
        subprocess.check_output([binary, "--no-daemon"]).decode('utf-8')
    except subprocess.CalledProcessError as e:
        print('Could not run %s, %s' % (binary, e.output))
        sys.exit(1)

    debug('--')

    # We've now finished updating the index now OR we completely failed
    print('Index now up to date!')

    # Check we have data in our index...
    db_query_have_files()


def index_shell():
    print('Starting shell... (type "exit" to finish)')
    print()

    os.system("/bin/bash")

# Environment / Clean up


def dbus_session_get_from_content(content):
    global dbus_session_address
    global dbus_session_pid

    if len(content) < 1:
        print('Content was empty ... can not get DBus session information from'
              ' empty string', file=sys.stderr)
        return False

    dbus_session_address = content.splitlines()[0]
    dbus_session_pid = int(content.splitlines()[1])

    err_msg = 'DBus session file was corrupt (%s), please remove "%s"'
    if dbus_session_address == '':
        print(err_msg % ("no address", dbus_session_file), file=sys.stderr)
        sys.exit(1)
    if dbus_session_pid < 0:
        print(err_msg % ("no PID", dbus_session_file), file=sys.stderr)
        sys.exit(1)

    return True


def dbus_session_file_get():
    try:
        with open(dbus_session_file, 'r') as f:
            content = f.read()
            return dbus_session_get_from_content(content)
    except FileNotFoundError as e:
        # Expect this if we have a new session to set up
        return False


def dbus_session_file_set():
    mkdir_p(os.environ['XDG_RUNTIME_DIR'])

    content = '%s\n%s' % (dbus_session_address, dbus_session_pid)
    with open(dbus_session_file, 'w') as f:
        f.write(content)


def environment_unset():
    debug('Cleaning up files ...')

    if not dbus_session_file == '':
        debug('  Removing DBus session file')
        os.unlink(dbus_session_file)

    debug('Cleaning up processes ...')

    if dbus_session_pid > 0:
        debug('  Killing DBus session')
        try:
            os.kill(dbus_session_pid, signal.SIGTERM)
        # (3, 'No such process') old python-schedutils incorrectly
        # raised SystemError
        except (SystemError, OSError):
            debug('    Process %d not found', dbus_session_pid)

    if not opts.update:
        return

    # FIXME: clean up tracker-store, can't use 'tracker daemon ...' for this,
    #        that kills everything it finds in /proc sadly.
    if store_pid > 0:
        debug('  Killing Tracker store')
        os.kill(store_pid, signal.SIGTERM)


def environment_set_and_add_path(env, prefix, suffix):
    new = os.path.join(prefix, suffix)

    if env in os.environ:
        existing = os.environ[env]
        full = '%s:%s' % (new, existing)
    else:
        full = new

    os.environ[env] = full


def environment_set():
    # Environment
    global dbus_session_address
    global dbus_session_pid
    global dbus_session_file
    global index_location_abs
    global default_debug_verbosity

    index_location_abs = os.path.abspath(opts.index_location)

    # Data
    os.environ['XDG_DATA_HOME'] = '%s/data/' % index_location_abs
    os.environ['XDG_CONFIG_HOME'] = '%s/config/' % index_location_abs
    os.environ['XDG_CACHE_HOME'] = '%s/cache/' % index_location_abs
    os.environ['XDG_RUNTIME_DIR'] = '%s/run/' % index_location_abs

    # Prefix - only set if non-standard
    if opts.prefix != default_prefix:
        environment_set_and_add_path('PATH', opts.prefix, 'bin')
        environment_set_and_add_path('LD_LIBRARY_PATH', opts.prefix, 'lib')
        environment_set_and_add_path('XDG_DATA_DIRS', opts.prefix, 'share')

    # Preferences
    os.environ['TRACKER_USE_CONFIG_FILES'] = 'yes'

    # if opts.debug:
    #     os.environ['TRACKER_USE_LOG_FILES'] = 'yes'

    if opts.debug:
        os.environ['G_MESSAGES_DEBUG'] = 'all'
        os.environ['TRACKER_VERBOSITY'] = '%d' % default_debug_verbosity
        os.environ['DBUS_VERBOSE'] = '1'
    else:
        os.environ['TRACKER_VERBOSITY'] = '0'

    debug('Using prefix location "%s"' % opts.prefix)
    debug('Using index location "%s"' % index_location_abs)

    # Ensure directory exists
    # DBus specific instance
    dbus_session_file = os.path.join(
        os.environ['XDG_RUNTIME_DIR'], 'dbus-session')

    if dbus_session_file_get() is False:
        output = subprocess.check_output(["dbus-daemon",
                                          "--session",
                                          "--print-address=1",
                                          "--print-pid=1",
                                          "--fork"]).decode('utf-8')

        dbus_session_get_from_content(output)
        dbus_session_file_set()
        debug('Using new D-Bus session with address "%s" with PID %d' %
              (dbus_session_address, dbus_session_pid))
    else:
        debug('Using existing D-Bus session from file "%s" with address "%s"'
              ' with PID %d' %
              (dbus_session_file, dbus_session_address, dbus_session_pid))

    # Important, other subprocesses must use our new bus
    os.environ['DBUS_SESSION_BUS_ADDRESS'] = dbus_session_address


def config_set():
    # Make sure File System miner is configured correctly
    config_dir = os.path.join(os.environ['XDG_CONFIG_HOME'], 'tracker')
    config_filename = os.path.join(config_dir, 'tracker-miner-fs.cfg')

    debug('Using config file "%s"' % config_filename)

    # Only update config if we're updating the database
    mkdir_p(config_dir)

    if not os.path.exists(config_filename):
        f = open(config_filename, 'w')
        f.write(config_template)
        f.close()

        debug('  Miner config file written')

    # Set content path
    config = configparser.ConfigParser()
    config.optionxform = str
    config.read(config_filename)

    if opts.content_locations_recursive:
        debug("Using content locations: %s" %
              opts.content_locations_recursive)
    if opts.content_locations_single:
        debug("Using non-recursive content locations: %s" %
              opts.content_locations_single)

    def locations_gsetting(locations):
        locations = [dir if dir.startswith('&') else os.path.abspath(dir)
                     for dir in locations]
        return GLib.Variant('as', locations).print_(False)

    if not config.has_section('General'):
        config.add_section('General')

    config.set('General', 'index-recursive-directories',
               locations_gsetting(opts.content_locations_recursive or []))
    config.set('General', 'index-single-directories',
               locations_gsetting(opts.content_locations_single or []))

    with open(config_filename, 'w') as f:
        config.write(f)


def link_to_mime_data():
    '''Create symlink to $XDG_DATA_HOME/mime in our custom data home dir.

    Mimetype detection seems to break horribly if the $XDG_DATA_HOME/mime
    directory is missing. Since we have to override the normal XDG_DATA_HOME
    path, we need to work around this by linking back to the real mime data.

    '''
    new_xdg_data_home = os.environ['XDG_DATA_HOME']
    old_mime_dir = os.path.join(original_xdg_data_home, 'mime')
    if os.path.exists(old_mime_dir):
        new_mime_dir = os.path.join(new_xdg_data_home, 'mime')
        if (not os.path.exists(new_mime_dir)
                and not os.path.islink(new_mime_dir)):
            mkdir_p(new_xdg_data_home)
            os.symlink(
                os.path.join(original_xdg_data_home, 'mime'), new_mime_dir)


# Entry point/start
if __name__ == "__main__":
    locale.setlocale(locale.LC_ALL, '')

    # Parse command line
    usage_oneline = '%s -i <DIR> -c <DIR> [OPTION...]' % (
        os.path.basename(sys.argv[0]))
    usage = '\n  %s - %s' % (usage_oneline, script_about)
    usage_invalid = 'Usage:\n  %s' % (usage_oneline)

    popt = optparse.OptionParser(usage)
    popt.add_option('-v', '--version',
                    action='count',
                    dest='version',
                    help='show version information')
    popt.add_option('-d', '--debug',
                    action='count',
                    dest='debug',
                    help='show additional debugging')
    popt.add_option('-p', '--prefix',
                    action='store',
                    metavar='PATH',
                    dest='prefix',
                    default=default_prefix,
                    help='use a non-standard prefix (default="%s")' %
                         (default_prefix))
    popt.add_option('-i', '--index',
                    action='store',
                    metavar='DIR',
                    dest='index_location',
                    help='directory storing the index')
    popt.add_option('-c', '--content',
                    action='append',
                    metavar='DIR',
                    dest='content_locations_recursive',
                    help='directory storing the content which is indexed (can '
                         'be specified multiple times)')
    popt.add_option('-C', '--content-non-recursive',
                    action='append',
                    metavar='DIR',
                    dest='content_locations_single',
                    help='directory storing the content which is indexed, '
                    'non-recursive variant (can be specified multiple times)')
    popt.add_option('-u', '--update',
                    action='count',
                    dest='update',
                    help='update index/database from content')
    popt.add_option('-l', '--list-files',
                    action='count',
                    dest='list_files',
                    help='list files indexed')
    popt.add_option('-s', '--shell',
                    action='count',
                    dest='shell',
                    help='start a shell with the environment set up')
    popt.add_option('--search',
                    action='store',
                    metavar='CRITERIA',
                    dest='search',
                    help='what content to look for in files')
    popt.add_option('-q', '--sparql',
                    action='store',
                    metavar='CRITERIA',
                    dest='sparql_query',
                    help='SPARQL query to execute')

    (opts, args) = popt.parse_args()

    if opts.version:
        print('%s %s\n%s\n' % (script_name, script_version, script_about))
        sys.exit(0)

    if not opts.index_location:
        if not opts.content_locations_recursive and not \
                opts.content_locations_single:
            print('Expected index (-i) or content (-c) locations to be '
                  'specified', file=sys.stderr)
            print(usage_invalid)
            sys.exit(1)

    if opts.update:
        if not opts.index_location or not (opts.content_locations_recursive or
                                           opts.content_locations_single):
            print('Expected index (-i) and content (-c) locations to be '
                  'specified', file=sys.stderr)
            print('These arguments are required to update the index databases',
                  file=sys.stderr)
            sys.exit(1)

    if ((opts.sparql_query or opts.search or opts.list_files or opts.shell)
            and not opts.index_location):
        print('Expected index location (-i) to be specified', file=sys.stderr)
        print('This arguments is required to use the content that has been '
              'indexed', file=sys.stderr)
        sys.exit(1)

    if (not opts.update
            and not opts.sparql_query
            and not opts.search
            and not opts.list_files
            and not opts.shell):
        print('No action specified (e.g. update (-u), shell (-s), '
              'list files (-l), etc)\n', file=sys.stderr)
        print('%s %s\n%s\n' % (script_name, script_version, script_about),
              file=sys.stderr)
        print(usage_invalid, file=sys.stderr)
        sys.exit(1)

    # Set up environment variables and foo needed to get started.
    environment_set()
    config_set()

    link_to_mime_data()

    try:
        if opts.update:
            index_update()

        if opts.list_files:
            db_query_list_files()

        if opts.shell:
            index_shell()
            sys.exit(0)

        if opts.search or opts.sparql_query:
            if not os.path.exists(index_location_abs):
                print('Can not query yet, index has not been created, see '
                      '--update or -u', file=sys.stderr)
                print(usage_invalid, file=sys.stderr)
                sys.exit(1)

        if opts.search:
            db_search(opts.search)

        if opts.sparql_query:
            db_sparql_query(opts.sparql_query)

    except KeyboardInterrupt:
        print('Handling Ctrl+C')

    environment_unset()



#
# Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

import configparser, os
import sys

config = configparser.ConfigParser()
config.readfp(sys.stdin)

print("<service>")
for section in config.sections():
   splitted = section.split (":")
   print(" <section prefix=\"" + splitted[0] + "\" name=\"" + splitted[1] + "\" fullname=\"" + section + "\">")
   for name, value in config.items (section):
     vsplit = value.split (":")
     print("\t<item name=\""+ name + "\" fullvalue=\"" + value+ "\">")
     if len (vsplit) > 1:
       print("\t\t<prefix>" + vsplit[0] + "</prefix>")
       print("\t\t<value>" + vsplit[1] + "</value>")
     else:
       print("\t\t<prefix />")
       print("\t\t<value>" + value + "</value>")
     print("\t</item>")
   print(" </section>")
print("</service>")



#!/usr/bin/python3
#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
# Copyright (C) 2019, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

"""
Stand-alone tests cases for the store, checking the statistics
are updated when different operations are executed on the store
"""
import time

import unittest as ut
from storetest import CommonTrackerStoreTest as CommonTrackerStoreTest

RDFS_RESOURCE = "rdfs:Resource"
NIE_IE = "nie:InformationElement"
RDFS_CLASS = "rdfs:Class"


class TrackerStoreStatisticsTests (CommonTrackerStoreTest):
    """
    Check initial statistics, add, remove, update content and check results stats
    """

    def __get_stats(self):
        results = {}
        for classname, count in self.tracker.get_stats():
            results[str(classname)] = int(count)
        return results

    def setUp(self):
        """
        Each test append to this list the used URIS, so they can be removed
        in the tearDown
        """
        self.clean_up_instances = []

    def tearDown(self):
        for uri in self.clean_up_instances:
            self.tracker.update("DELETE { <%s> a rdfs:Resource. }" % (uri))
        self.clean_up_instances = []
        time.sleep(1)

    def test_stats_01_insert_base_class(self):
        self.clean_up_instances.append("test://stats-01")

        old_stats = self.__get_stats()
        self.tracker.update(
            "INSERT { <test://stats-01> a nie:InformationElement. }")
        new_stats = self.__get_stats()

        increased_classes = [NIE_IE, RDFS_RESOURCE]

        for k, v in new_stats.items():
            if k in increased_classes:
                self.assertEqual(old_stats[k] + 1, new_stats[k])
            else:
                self.assertEqual(old_stats[k], new_stats[k],
                                 "Class %s should have the same instances" % k)

    def test_stats_02_insert_deep_class(self):
        self.clean_up_instances.append("test://stats-02")
        old_stats = self.__get_stats()
        self.tracker.update("INSERT { <test://stats-02> a nmm:Photo. }")
        new_stats = self.__get_stats()

        increased_classes = [NIE_IE, RDFS_RESOURCE]
        new_classes = ["nmm:Photo", "nfo:Visual", "nfo:Image", "nfo:Media"]

        # There were no instances of those classes before, check they are now
        for c in new_classes:
            self.assertIn(c, new_stats)

        for k, v in new_stats.items():
            if k in increased_classes:
                self.assertEqual(old_stats[k] + 1, new_stats[k])
            elif k in new_classes:
                # This classes could exists previous or not!
                if k in old_stats:
                    self.assertEqual(old_stats[k] + 1, new_stats[k])
                else:
                    self.assertEqual(new_stats[k], 1)
            else:
                self.assertEqual(old_stats[k], new_stats[k])

    def test_stats_03_delete_deep_class(self):
        self.clean_up_instances.append("test://stats-03")
        self.tracker.update("INSERT { <test://stats-03> a nmm:Photo. }")

        old_stats = self.__get_stats()
        self.tracker.update("DELETE { <test://stats-03> a rdfs:Resource. }")
        new_stats = self.__get_stats()

        decreased_classes = [NIE_IE, RDFS_RESOURCE]
        # These classes could have no instance
        no_instances_classes = ["nmm:Photo",
                                "nfo:Visual", "nfo:Image", "nfo:Media"]

        for c in no_instances_classes:
            if (old_stats[c] == 1):
                self.assertNotIn(c, new_stats)
            else:
                self.assertEqual(old_stats[c] - 1, new_stats[c])

        for k, v in new_stats.items():
            if k in decreased_classes:
                self.assertEqual(old_stats[k] - 1, new_stats[k])
            else:
                self.assertEqual(old_stats[k], new_stats[k])

if __name__ == "__main__":
    ut.main()



#!/usr/bin/env python
import gi
from gi.repository import Tracker

conn = Tracker.SparqlConnection.get (None)
cursor = conn.query ("SELECT ?u WHERE { ?u a nie:InformationElement. }", None)

while (cursor.next (None)):
    print cursor.get_string (0)



#!/usr/bin/python3
#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
# Copyright (C) 2019, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
"""
Replicate the behaviour of the miner inserting information in the store.
"""
import unittest as ut
from storetest import CommonTrackerStoreTest as CommonTrackerStoreTest


class TestMinerInsertBehaviour (CommonTrackerStoreTest):
    """
    Mimic the behaviour of the miner, removing the previous information of the resource
    and inserting a new one.
    """

    def test_miner_unique_insertion(self):
        """
        We actually can't test tracker-miner-fs, so we mimick its behavior in this test
        1. Insert one resource
        2. Update it like tracker-miner-fs does
        3. Check there isn't any duplicate
        4. Clean up
        """

        resource = 'graph://test/resource/1'

        insert_sparql = """
        DELETE { ?r a rdfs:Resource } WHERE { GRAPH <graph://test/resource/1> { ?r a rdfs:Resource } }
        INSERT INTO <graph://test/resource/1> {
           _:resource a nie:DataObject ;
                      nie:url "%s" .
        }
        """ % resource

        select_sparql = """
        SELECT ?u { ?u nie:url "%s" }
        """ % resource

        delete_sparql = """
        DELETE { ?r a rdfs:Resource } WHERE { GRAPH <graph://test/resource/1> { ?r a rdfs:Resource } }
        """

        ''' First insertion '''
        self.tracker.update(insert_sparql)

        results = self.tracker.query(select_sparql)
        self.assertEqual(len(results), 1)

        ''' Second insertion / update '''
        self.tracker.update(insert_sparql)

        results = self.tracker.query(select_sparql)
        self.assertEqual(len(results), 1)

        ''' Clean up '''
        self.tracker.update(delete_sparql)

        results = self.tracker.query(select_sparql)
        self.assertEqual(len(results), 0)


if __name__ == '__main__':
    ut.main()



#
# Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

import configparser, os
import sys
import getopt

def usage():
  print("Usage: python service2rdf-xml.py --metadata=ONTOLOGY.metadata --service=ONTOLOGY.service [--uri=URI]")

def main():
  try:
     uri = ""
     metadataf = ""
     servicef = ""

     opts, args = getopt.getopt(sys.argv[1:], "hu:va:vm:vs:v", ["help", "uri=", "metadata=", "service="])

     for o, a in opts:
         if o in ("-u", "--uri"):
           uri = a
         elif o in ("-m", "--metadata"):
           metadataf = a
         elif o in ("-s", "--service"):
           servicef = a
         elif o in ("-h", "--help"):
           usage ()
           sys.exit()

     if uri == "":
       uri = "http://live.gnome.org/Tracker/XMLSchema"

     if metadataf == "" or servicef == "":
       usage ()
       sys.exit ()

     service = configparser.ConfigParser()
     service.readfp(open(servicef))

     metadata = configparser.ConfigParser()
     metadata.readfp(open(metadataf))

     print("<rdf:RDF")
     print("  xmlns:nid3=\"http://www.semanticdesktop.org/ontologies/2007/05/10/nid3#\"")
     print("  xmlns:nfo=\"http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#\"")
     print("  xmlns:nmo=\"http://www.semanticdesktop.org/ontologies/2007/03/22/nmo#\"")
     print("  xmlns:nie=\"http://www.semanticdesktop.org/ontologies/2007/01/19/nie#\"")
     print("  xmlns:exif=\"http://www.kanzaki.com/ns/exif#\"")
     print("  xmlns:nao=\"http://www.semanticdesktop.org/ontologies/2007/08/15/nao#\"")
     print("  xmlns:rdfs=\"http://www.w3.org/2000/01/rdf-schema#\"")
     print("  xmlns:protege=\"http://protege.stanford.edu/system#\"")
     print("  xmlns:dcterms=\"http://purl.org/dc/terms/\"")
     print("  xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"")
     print("  xmlns:ncal=\"http://www.semanticdesktop.org/ontologies/2007/04/02/ncal#\"")
     print("  xmlns:xsd=\"http://www.w3.org/2001/XMLSchema#\"")
     print("  xmlns:nrl=\"http://www.semanticdesktop.org/ontologies/2007/08/15/nrl#\"")
     print("  xmlns:pimo=\"http://www.semanticdesktop.org/ontologies/2007/11/01/pimo#\"")
     print("  xmlns:geo=\"http://www.w3.org/2003/01/geo/wgs84_pos#\"")
     print("  xmlns:tmo=\"http://www.semanticdesktop.org/ontologies/2008/05/20/tmo#\"")
     print("  xmlns:dc=\"http://purl.org/dc/elements/1.1/\"")
     print("  xmlns:nco=\"http://www.semanticdesktop.org/ontologies/2007/03/22/nco#\"")
     print("  xmlns:nexif=\"http://www.semanticdesktop.org/ontologies/2007/05/10/nexif#\">")

     print("")

     for klass in service.sections():
        splitted = klass.split (":")
        print("\t<rdfs:Class rdf:about=\"" + uri + "/" + splitted[0] + "#" + splitted[1] + "\">")
        print("\t\t<rdfs:label>" + splitted[1] + "</rdfs:label>")

        for name, value in service.items (klass):
           if name == "SuperClasses":
             vsplit = value.split (";")
             for val in vsplit:
               vvsplit = val.split (":");
               print("\t\t<rdfs:subClassOf>")
               print("\t\t\t<rdfs:Class rdf:about=\"" + uri + "/" +vvsplit[0] + "#" + vvsplit[1] + "\"/>")
               print("\t\t</rdfs:subClassOf>")
        print("\t</rdfs:Class>")

     for mdata in metadata.sections():
        splitted = mdata.split (":")
        print("\t<rdf:Property rdf:about=\"" + uri + "#" + splitted[1] + "\">")
        print("\t\t<rdfs:label>" + splitted[1] + "</rdfs:label>")

        for name, value in metadata.items (mdata):
           if name == "datatype":
             print("\t\t<rdfs:range rdf:resource=\"" + uri + "#" + value + "\"/>")

           if name == "domain":
            vvsplit = value.split (":")
            print("\t\t<rdfs:domain rdf:resource=\"" + uri + "/" +vvsplit[0] + "#" + vvsplit[1] + "\"/>")

           if name == "parent":
            print("\t\t<rdfs:subPropertyOf rdf:resource=\"" + uri + "#" + value.split (":")[1] + "\"/>")

           if name == "weight":
            print("\t\t<rdfs:comment>Weight is " + value + "</rdfs:comment>")

        print("\t</rdf:Property>")

     print("</rdf:RDF>")
  except getopt.GetoptError as err:
     print(str(err))
     usage ()
     sys.exit(2)

if __name__ == "__main__":
    main()




#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
# Copyright (C) 2018, 2019, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

import os
import time
import unittest as ut

import trackertestutils.helpers
import configuration as cfg


class CommonTrackerStoreTest (ut.TestCase):
    """
    Common superclass for tests that just require a fresh store running

    Note that the store is started per test-suite, not per test.
    """

    @classmethod
    def setUpClass(self):
        extra_env = {'LC_COLLATE': 'en_GB.utf8'}

        self.tracker = trackertestutils.helpers.StoreHelper(cfg.TRACKER_STORE_PATH)
        self.tracker.start(extra_env=extra_env)

    @classmethod
    def tearDownClass(self):
        self.tracker.stop()



#!/usr/bin/python3
#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
# Copyright (C) 2019, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

"""
Stand-alone tests cases for the store, inserting, removing information
in pure sparql and checking that the data is really there
"""
import sys
import time
import random
import datetime

import unittest as ut
from storetest import CommonTrackerStoreTest as CommonTrackerStoreTest


class TrackerStoreInsertionTests (CommonTrackerStoreTest):
    """
    Insert single and multiple-valued properties, dates (ok and broken)
    and check the results
    """

    def test_insert_01(self):
        """
        Simple insert of two triplets.

        1. Insert a InformationElement with title.
        2. TEST: Query the title of that information element
        3. Remove the InformationElement to keep everything as it was before
        """

        uri = "tracker://test_insert_01/" + str(random.randint(0, 100))
        insert = """
                INSERT { <%s> a nie:InformationElement;
                        nie:title \"test_insert_01\". }
                """ % (uri)
        self.tracker.update(insert)

        """ verify the inserted item """
        query = """
                SELECT ?t WHERE {
                <%s> a nie:InformationElement ;
                nie:title ?t .
                }
                """ % (uri)
        results = self.tracker.query(query)

        self.assertEqual(str(results[0][0]), "test_insert_01")

        """ delete the inserted item """
        delete = """
                DELETE { <%s> a rdfs:Resource. }
                """ % (uri)
        self.tracker.update(delete)

    def test_insert_02(self):
        """
        Insert of a bigger set of triplets (linking two objects)
        """

        self.tracker.update("""
                INSERT {
                <urn:uuid:bob-dylan> a nmm:Artist;
                   nmm:artistName 'Bob Dylan'.

                <file:///a/b/c/10_song3.mp3> a nmm:MusicPiece, nfo:FileDataObject;
                   nfo:fileName 'subterranean-homesick-blues.mp3';
                   nfo:fileLastModified '2008-10-23T13:47:02' ;
                   nfo:fileCreated '2008-12-16T12:41:20' ;
                   nfo:fileSize 17630 ;
                   nfo:duration 219252 ;
                   nie:title 'Subterranean homesick blues';
                   nmm:performer <urn:uuid:bob-dylan>.
                   }
                   """)

        QUERY = """
                SELECT ?uri ?title ?length  WHERE {
                    ?uri a nmm:MusicPiece ;
                         nmm:performer <urn:uuid:bob-dylan> ;
                         nie:title ?title ;
                         nfo:duration ?length .
                }
                """

        result = self.tracker.query(QUERY)
        self.assertEqual(len(result), 1)
        self.assertEqual(len(result[0]), 3)  # uri, title, length
        self.assertEqual(result[0][0], "file:///a/b/c/10_song3.mp3")
        self.assertEqual(result[0][1], "Subterranean homesick blues")
        self.assertEqual(result[0][2], "219252")

        self.tracker.update ("""
                DELETE {
                   <urn:uuid:bob-dylan> a rdfs:Resource.
                   <file:///a/b/c/10_song3.mp3> a rdfs:Resource.
                }
                """)

    def test_insert_03(self):
        """
        Checking all the values are inserted
        """

        self.tracker.update("""
                INSERT {
                <urn:uuid:7646004> a nmm:Artist;
                    nmm:artistName 'John Lennon' .

                <urn:uuid:123123123> a nmm:MusicAlbum ;
                    nie:title 'Imagine' .

                <file:///a/b/c/imagine.mp3> a nmm:MusicPiece, nfo:FileDataObject;
                    nfo:fileName 'imagine.mp3';
                    nfo:fileCreated '2008-12-16T12:41:20';
                    nfo:fileLastModified '2008-12-23T13:47:02' ;
                    nfo:fileSize 17630;
                    nmm:musicAlbum <urn:uuid:123123123>;
                    nmm:trackNumber '11';
                    nfo:duration 219252;
                    nmm:performer <urn:uuid:7646004>.
                    }

                    """)

        QUERY = """
                SELECT ?artist ?length ?trackN ?album ?size ?flm ?fc ?filename  WHERE {
                    <file:///a/b/c/imagine.mp3> a nmm:MusicPiece ;
                        nmm:performer ?x ;
                        nfo:duration ?length ;
                        nmm:trackNumber ?trackN ;
                        nmm:musicAlbum ?y ;
                        nfo:fileSize ?size ;
                        nfo:fileLastModified ?flm ;
                        nfo:fileCreated ?fc ;
                        nfo:fileName ?filename.

                    ?x nmm:artistName ?artist .
                    ?y nie:title ?album.
                    }
                    """
        result = self.tracker.query(QUERY)

        self.assertEqual(len(result), 1)
        self.assertEqual(len(result[0]), 8)
        self.assertEqual(result[0][0], "John Lennon")
        self.assertEqual(result[0][1], "219252")
        self.assertEqual(result[0][2], "11")
        self.assertEqual(result[0][3], "Imagine")
        self.assertEqual(result[0][4], "17630")
        # FIXME Tracker returns this translated to the current timezone
        #self.assertEquals (result[0][5], "2008-12-23T11:47:02Z")
        #self.assertEquals (result[0][6], "2008-12-16T10:41:20Z")
        self.assertEqual(result[0][7], "imagine.mp3")

        self.tracker.update ("""
                DELETE {
                   <urn:uuid:123123123> a rdfs:Resource .
                }

                DELETE {
                  <file:///a/b/c/imagine.mp3> a rdfs:Resource.
                }
                """)

    def test_insert_04(self):
        """
        Insert, delete same single valued properties multiple times.
        """
        for i in range(0, 3):
            # Delete single valued properties of music file.
            self.tracker.update("""
                        DELETE {
                          <test://instance-1> nie:usageCounter ?v
                        } WHERE {
                          <test://instance-1> nie:usageCounter ?v .
                        }
                        DELETE {
                          <test://instance-1> nie:contentAccessed ?w .
                        } WHERE {
                          <test://instance-1> nie:contentAccessed ?w .
                        }
                        """)

            # Insert the same single valued properties of music file.
            self.tracker.update("""
                        INSERT {
                           <test://instance-1> a nmm:MusicPiece, nfo:FileDataObject;
                           nie:usageCounter '%d';
                           nie:contentAccessed '2000-01-01T00:4%d:47Z' .
                        }""" % (i, i))

            # Query for the property values and verify whether the last change
            # is applied.
            result = self.tracker.query ("""
                          SELECT ?playcount ?date WHERE {
                             <test://instance-1> a nmm:MusicPiece ;
                                 nie:usageCounter ?playcount ;
                                 nie:contentAccessed ?date.
                          }""")

            self.assertEqual(len(result), 1)
            self.assertEqual(len(result[0]), 2)
            self.assertEqual(int(result[0][0]), i)
            self.assertEqual(result[0][1], "2000-01-01T00:4%d:47Z" % (i))

        self.tracker.update ("""
                DELETE { <test://instance-1> a rdfs:Resource. }
                """)

    def test_insert_05(self):
        """
        Insert or replace, single valued properties multiple times.
        """
        for i in range(0, 3):
            # Insert the same single valued properties of music file.
            self.tracker.update("""
                        INSERT OR REPLACE {
                           <test://instance-1> a nmm:MusicPiece, nfo:FileDataObject;
                           nie:usageCounter '%d';
                           nie:contentAccessed '2000-01-01T00:4%d:47Z' .
                        }""" % (i, i))

            # Query for the property values and verify whether the last change
            # is applied.
            result = self.tracker.query ("""
                          SELECT ?playcount ?date WHERE {
                             <test://instance-1> a nmm:MusicPiece ;
                                 nie:usageCounter ?playcount ;
                                 nie:contentAccessed ?date.
                          }""")

            self.assertEqual(len(result), 1)
            self.assertEqual(len(result[0]), 2)
            self.assertEqual(int(result[0][0]), i)
            self.assertEqual(result[0][1], "2000-01-01T00:4%d:47Z" % (i))

        self.tracker.update ("""
                DELETE { <test://instance-1> a rdfs:Resource. }
                """)

    def test_insert_06(self):
        """
        Insert or replace, single and multi valued properties multiple times.
        """
        for i in range(0, 3):
            # Insert the same single valued properties and insert multi valued
            # properties at the same time
            self.tracker.update("""
                        INSERT OR REPLACE {
                           <test://instance-2> a nie:InformationElement;
                           nie:title '%d';
                           nie:keyword '%d'
                        }""" % (i, i))

            # Query for the property values and verify whether the last change
            # is applied.
            result = self.tracker.query ("""
                          SELECT ?t ?k WHERE {
                             <test://instance-2> nie:title ?t ;
                                 nie:keyword ?k 
                          }""")

        self.assertEqual(len(result), 3)
        self.assertEqual(len(result[0]), 2)
        self.assertEqual(result[0][0], "%d" % i)
        self.assertEqual(result[0][1], "0")

        self.assertEqual(result[1][0], "%d" % i)
        self.assertEqual(result[1][1], "1")

        self.assertEqual(result[2][0], "%d" % i)
        self.assertEqual(result[2][1], "2")

        self.tracker.update ("""
                DELETE { <test://instance-2> a rdfs:Resource. }
                """)

    def test_insert_07(self):
        """
        Insert or replace, single and multi valued properties with domain errors.
        """

        try:
            INSERT_SPARQL = """INSERT OR REPLACE { <test://instance-3> nie:title 'test' }"""
            self.tracker.update(INSERT_SPARQL)
        except:
            pass

        INSERT_SPARQL = """INSERT OR REPLACE { <test://instance-4> a nie:DataSource }"""
        self.tracker.update(INSERT_SPARQL)

        try:
            INSERT_SPARQL = """INSERT OR REPLACE { <test://instance-5> nie:rootElementOf <test://instance-4> }"""
            self.tracker.update(INSERT_SPARQL)
        except:
            pass

        INSERT_SPARQL = """INSERT OR REPLACE { <test://instance-5> a nie:InformationElement ; nie:rootElementOf <test://instance-4> }"""
        self.tracker.update(INSERT_SPARQL)

        self.tracker.update ("""
                DELETE { <test://instance-4> a rdfs:Resource. }
                """)

        self.tracker.update ("""
                DELETE { <test://instance-5> a rdfs:Resource. }
                """)

    def test_insert_08(self):
        """
        Insert or replace, single and multi valued properties with graphs
        """

        INSERT_SPARQL = """INSERT { GRAPH <test://graph-1> { <test://instance-6> a nie:InformationElement ; nie:title 'title 1' } }"""
        self.tracker.update(INSERT_SPARQL)

        INSERT_SPARQL = """INSERT { GRAPH <test://graph-2> { <test://instance-6> nie:title 'title 1' } }"""
        self.tracker.update(INSERT_SPARQL)

        result = self.tracker.query ("""
                          SELECT ?g ?t WHERE { GRAPH ?g {
                             <test://instance-6> nie:title ?t
                           } }""")

        self.assertEqual(len(result), 1)
        self.assertEqual(len(result[0]), 2)
        self.assertEqual(result[0][0], "test://graph-1")  # Yes, indeed
        self.assertEqual(result[0][1], "title 1")

        INSERT_SPARQL = """INSERT OR REPLACE { GRAPH <test://graph-2> { <test://instance-6> nie:title 'title 1' } }"""
        self.tracker.update(INSERT_SPARQL)

        result = self.tracker.query ("""
                          SELECT ?g ?t WHERE { GRAPH ?g {
                             <test://instance-6> nie:title ?t
                           } }""")

        self.assertEqual(len(result), 1)
        self.assertEqual(len(result[0]), 2)
        self.assertEqual(result[0][0], "test://graph-2")  # Yup, that's right
        self.assertEqual(result[0][1], "title 1")

        INSERT_SPARQL = """INSERT OR REPLACE { GRAPH <test://graph-3> { <test://instance-6> nie:title 'title 2' } }"""
        self.tracker.update(INSERT_SPARQL)

        result = self.tracker.query ("""
                          SELECT ?g ?t WHERE { GRAPH ?g {
                             <test://instance-6> nie:title ?t
                           } }""")

        self.assertEqual(len(result), 1)
        self.assertEqual(len(result[0]), 2)
        self.assertEqual(result[0][0], "test://graph-3")
        self.assertEqual(result[0][1], "title 2")

        self.tracker.update ("""
                DELETE { <test://instance-6> a rdfs:Resource. }
                """)

    def __insert_valid_date_test(self, datestring, year, month, day, hours, minutes, seconds, timezone):
        """
        Insert a property with datestring value, retrieve its components and validate against
        the expected results (all the other parameters)
        """
        testId = random.randint(10, 1000)
        self.tracker.update ("""
                INSERT {
                   <test://instance-insert-date-%d> a nie:InformationElement;
                        nie:informationElementDate '%s'.
                }
                """ % (testId, datestring))

        result = self.tracker.query ("""
                SELECT    fn:year-from-dateTime (?v)
                          fn:month-from-dateTime (?v)
                          fn:day-from-dateTime (?v)
                          fn:hours-from-dateTime (?v)
                          fn:minutes-from-dateTime (?v)
                          fn:seconds-from-dateTime (?v)
                          fn:timezone-from-dateTime (?v)
                WHERE {
                  <test://instance-insert-date-%d> a nie:InformationElement;
                        nie:informationElementDate ?v .
                }
                 """ % (testId))
        try:
            self.assertEqual(len(result), 1)
            self.assertEqual(len(result[0]), 7)
            self.assertEqual(result[0][0], year)
            self.assertEqual(result[0][1], month)
            self.assertEqual(result[0][2], day)
            self.assertEqual(result[0][3], hours)
            self.assertEqual(result[0][4], minutes)
            self.assertEqual(result[0][5], seconds)
            # FIXME To validate this we need to take into account the locale
            # self.assertEquals (result[0][7], timezone)
        finally:
            self.tracker.update ("""
                        DELETE { <test://instance-insert-date-%d> a rdfs:Resource. }
                        """ % (testId))

    """Date-Time storage testing """

    def test_insert_date_01(self):
        """
        1. Insert a InformationElement with date having local timezone info.
        2. TEST: Query and verify the various componentes of date
        """
        self.__insert_valid_date_test("2004-05-06T13:14:15+0400",
                                      "2004", "05", "06", "13", "14", "15", "14400")

    def test_insert_date_02(self):
        """
        1. Insert a InformationElement with date ending with "Z" in TZD.
        2. TEST: Query and verify the various componentes of date
        """
        self.__insert_valid_date_test("2004-05-06T13:14:15Z",
                                      "2004", "05", "06", "13", "14", "15", "0")

    def test_insert_date_03(self):
        """
        1. Insert a InformationElement with date ending with no TZD.
        2. TEST: Query and verify the various componentes of date
        """
        self.__insert_valid_date_test("2004-05-06T13:14:15",
                                      "2004", "05", "06", "13", "14", "15", "10800")  # HEL timezone?

    #@ut.skipIf (1, "It times out in the daemon. Investigate")
    def test_insert_date_04(self):
        """
        1. Insert a InformationElement with date having local timezone info
           with some minutes in it.
        2. TEST: Query and verify the various componentes of date
        """
        self.__insert_valid_date_test("2004-05-06T13:14:15+0230",
                                      "2004", "05", "06", "13", "14", "15", "9000")

    #@ut.skipIf (1, "It times out in the daemon. Investigate")
    def __test_insert_date_05(self):
        """
         1. Insert a InformationElement with date having local timezone info in negative.
         2. TEST: Query and verify the various componentes of date
         """
        self.__insert_valid_date_test("2004-05-06T13:14:15-0230",
                                      "2004", "05", "06", "13", "14", "15", "-9000")

    def __insert_invalid_date_test(self, datestring):
        self.assertRaises (Exception, self.tracker.update, """
                        INSERT {
                           <test://instance-insert-invalid-date-01> a nie:InformationElement;
                              nie:informationElementDate '204-05-06T13:14:15+0400'.
                        }
                        """)

        result = self.tracker.query ("""
                SELECT    fn:year-from-dateTime (?v)
                          fn:month-from-dateTime (?v)
                          fn:day-from-dateTime (?v)
                          fn:hours-from-dateTime (?v)
                          fn:minutes-from-dateTime (?v)
                          fn:seconds-from-dateTime (?v)
                          fn:timezone-from-dateTime (?v)
                WHERE {
                   <test://instances-insert-invalid-date-01> a nie:InformationElement ;
                        nie:informationElementDate ?v .
                }
                """)
        self.assertEqual(len(result), 0)

        #@ut.skipIf (1, "It times out in the daemon. Investigate")
    def test_insert_invalid_date_01(self):
        """
        1. Insert a InformationElement with invalid year in date.
        2. TEST: Query and verify the various componentes of date
        """
        self.__insert_invalid_date_test("204-05-06T13:14:15+0400")

        #@ut.skipIf (1, "It times out in the daemon. Investigate")
    def test_insert_invalid_date_02(self):
        """
        1. Insert a InformationElement with date without time.
        2. TEST: Query and verify the various componentes of date
        """
        self.__insert_invalid_date_test("2004-05-06")

        #@ut.skipIf (1, "It times out in the daemon. Investigate")
    def test_insert_invalid_date_03(self):
        """
        1. Insert a InformationElement with date without time but only the "T" separator.
        """
        self.__insert_invalid_date_test("2004-05-06T")

        #@ut.skipIf (1, "It times out in the daemon. Investigate")
    def test_insert_invalid_date_04(self):
        """
        1. Insert a InformationElement with date without time but only the "T" separator.
        """
        self.__insert_invalid_date_test("2004-05-06T1g:14:15-0200")

    def test_insert_duplicated_url_01(self):
        """
        1. Insert a FileDataObject with a known nie:url, twice
        """

        url = "file:///some/magic/path/here"

        insert = """
                INSERT {
                   _:tag a nfo:FileDataObject;
                         nie:url '%s'.
                }
                """ % (url)

        # First insert should go ok
        self.tracker.update(insert)
        # Second insert should not be ok
        try:
            self.tracker.update(insert)
        except Exception:
            pass

        # Only 1 element must be available with the given nie:url
        select = """
                SELECT ?u WHERE { ?u nie:url \"%s\" }
                """ % (url)
        self.assertEqual(len(self.tracker.query(select)), 1)

        # Cleanup
        self.tracker.update ("""
                DELETE { ?u a rdfs:Resource } WHERE { ?u a rdfs:Resource ; nie:url '%s' }
                """ % (url))

    def test_insert_replace_null(self):
        """
        Insert or replace, with null
        """

        self.tracker.update(
            """INSERT { <test://instance-null> a nie:DataObject, nie:InformationElement }""")
        self.tracker.update(
            """INSERT { <test://instance-ds1> a nie:DataSource  }""")
        self.tracker.update(
            """INSERT { <test://instance-ds2> a nie:DataSource  }""")
        self.tracker.update(
            """INSERT { <test://instance-ds3> a nie:DataSource  }""")
        self.tracker.update(
            """INSERT { <test://instance-null> nie:dataSource <test://instance-ds1>, <test://instance-ds2>, <test://instance-ds3> }""")

        # null upfront, reset of list, rewrite of new list
        self.tracker.update(
            """INSERT OR REPLACE { <test://instance-null> nie:dataSource null, <test://instance-ds1>, <test://instance-ds2> }""")
        result = self.tracker.query(
            """SELECT ?ds WHERE { <test://instance-null> nie:dataSource ?ds }""")
        self.assertEqual(len(result), 2)
        self.assertEqual(len(result[0]), 1)
        self.assertEqual(len(result[1]), 1)
        self.assertEqual(result[0][0], "test://instance-ds1")
        self.assertEqual(result[1][0], "test://instance-ds2")

        # null upfront, reset of list, rewrite of new list, second test
        self.tracker.update(
            """INSERT OR REPLACE { <test://instance-null> nie:dataSource null, <test://instance-ds1>, <test://instance-ds2>, <test://instance-ds3> }""")
        result = self.tracker.query(
            """SELECT ?ds WHERE { <test://instance-null> nie:dataSource ?ds }""")
        self.assertEqual(len(result), 3)
        self.assertEqual(len(result[0]), 1)
        self.assertEqual(len(result[1]), 1)
        self.assertEqual(len(result[2]), 1)
        self.assertEqual(result[0][0], "test://instance-ds1")
        self.assertEqual(result[1][0], "test://instance-ds2")
        self.assertEqual(result[2][0], "test://instance-ds3")

        # null in the middle, rewrite of new list
        self.tracker.update(
            """INSERT OR REPLACE { <test://instance-null> nie:dataSource <test://instance-ds1>, null, <test://instance-ds2>, <test://instance-ds3> }""")
        result = self.tracker.query(
            """SELECT ?ds WHERE { <test://instance-null> nie:dataSource ?ds }""")
        self.assertEqual(len(result), 2)
        self.assertEqual(len(result[0]), 1)
        self.assertEqual(len(result[1]), 1)
        self.assertEqual(result[0][0], "test://instance-ds2")
        self.assertEqual(result[1][0], "test://instance-ds3")

        # null at the end
        self.tracker.update(
            """INSERT OR REPLACE { <test://instance-null> nie:dataSource <test://instance-ds1>, <test://instance-ds2>, <test://instance-ds3>, null }""")
        result = self.tracker.query(
            """SELECT ?ds WHERE { <test://instance-null> nie:dataSource ?ds }""")
        self.assertEqual(len(result), 0)

        # Multiple nulls
        self.tracker.update(
            """INSERT OR REPLACE { <test://instance-null> nie:dataSource null, <test://instance-ds1>, null, <test://instance-ds2>, <test://instance-ds3> }""")
        result = self.tracker.query(
            """SELECT ?ds WHERE { <test://instance-null> nie:dataSource ?ds }""")
        self.assertEqual(len(result), 2)
        self.assertEqual(len(result[0]), 1)
        self.assertEqual(len(result[1]), 1)
        self.assertEqual(result[0][0], "test://instance-ds2")
        self.assertEqual(result[1][0], "test://instance-ds3")

        self.tracker.update(
            """DELETE { <test://instance-null> a rdfs:Resource. }""")
        self.tracker.update(
            """DELETE { <test://instance-ds1> a rdfs:Resource. }""")
        self.tracker.update(
            """DELETE { <test://instance-ds2> a rdfs:Resource. }""")
        self.tracker.update(
            """DELETE { <test://instance-ds3> a rdfs:Resource. }""")


class TrackerStoreDeleteTests (CommonTrackerStoreTest):
    """
    Use DELETE in Sparql and check the information is actually removed
    """

    def test_delete_01(self):
        """
        Insert triples and Delete a triple. Verify the deletion with a query
        """

        # first insert
        self.tracker.update ("""
                INSERT {
                   <urn:uuid:7646001> a nco:Contact;
                            nco:fullname 'Artist_1_delete'.
                   <test://instance-test-delete-01> a nmm:MusicPiece, nfo:FileDataObject;
                            nfo:fileName '11_song_del.mp3';
                            nfo:genre 'Classic delete';
                            nmm:musicAlbum '1_Album_delete';
                            nmm:performer <urn:uuid:7646001>.
                }
                """)

        # verify the insertion
        result = self.tracker.query ("""
                SELECT ?u WHERE {
                    ?u a nmm:MusicPiece ;
                         nfo:genre 'Classic delete' .
                }
                """)
        self.assertEqual(len(result), 1)
        self.assertEqual(len(result[0]), 1)
        self.assertEqual(result[0][0], "test://instance-test-delete-01")

        # now delete
        self.tracker.update("""
                DELETE {
                  <test://instance-test-delete-01> a rdfs:Resource.
                }
                """)

        # Check the instance is not there
        result = self.tracker.query ("""
                SELECT ?u WHERE {
                    ?u a nmm:MusicPiece ;
                         nfo:genre 'Classic delete' .
                }
                """)
        self.assertEqual(len(result), 0)

    def test_delete_02(self):
        """
        Delete a MusicAlbum and count the album

        1. add a music album.
        2. count the number of albums
        3. delete an album
        2. count the number of albums
        """

        initial = self.tracker.count_instances("nmm:MusicAlbum")

        """Add a music album """
        self.tracker.update ("""
                INSERT {
                   <test://instance-delete-02> a nmm:MusicAlbum;
                           nie:title '06_Album_delete'.
                }
                """)

        after_insert = self.tracker.count_instances("nmm:MusicAlbum")
        self.assertEqual(initial + 1, after_insert)

        """Delete the added music album """
        self.tracker.update("""
                DELETE {
                  <test://instance-delete-02> a nmm:MusicAlbum.
                }
                """)

        """get the count of music albums"""
        after_removal = self.tracker.count_instances("nmm:MusicAlbum")

        self.assertEqual(after_removal, initial)


class TrackerStoreBatchUpdateTest (CommonTrackerStoreTest):
    """
    Insert data using the BatchSparqlUpdate method in the store
    """

    def test_batch_insert_01(self):
        """
        batch insertion of 100 contacts:
        1. insert 100 contacts.
        2. delete the inserted contacts.
        """
        NUMBER_OF_TEST_CONTACTS = 3

        # query no. of existing contacts. (predefined instances in the DB)
        count_before_insert = self.tracker.count_instances("nco:PersonContact")

        # insert contacts.
        CONTACT_TEMPLATE = """
                   <test://instance-contact-%d> a nco:PersonContact ;
                      nco:nameGiven 'Contact-name %d';
                      nco:nameFamily 'Contact-family %d';
                      nie:generator 'test-instance-to-remove' ;
                      nco:contactUID '%d';
                      nco:hasPhoneNumber <tel:%s> .
                """

        global contact_list
        contact_list = []

        def complete_contact(contact_template):
            random_phone = "".join([str(random.randint(0, 9))
                                    for i in range(0, 9)])
            contact_counter = random.randint(0, 10000)

            # Avoid duplicates
            while contact_counter in contact_list:
                contact_counter = random.randint(0, 10000)
            contact_list.append(contact_counter)

            return contact_template % (contact_counter,
                                       contact_counter,
                                       contact_counter,
                                       contact_counter,
                                       random_phone)

        contacts = list(
            map(complete_contact, [CONTACT_TEMPLATE] * NUMBER_OF_TEST_CONTACTS))
        INSERT_SPARQL = "\n".join(["INSERT {"] + contacts + ["}"])
        self.tracker.batch_update(INSERT_SPARQL)

        # Check all instances are in
        count_after_insert = self.tracker.count_instances("nco:PersonContact")
        self.assertEqual(count_before_insert +
                         NUMBER_OF_TEST_CONTACTS, count_after_insert)

        """ Delete the inserted contacts """
        DELETE_SPARQL = """
                DELETE {
                  ?x a rdfs:Resource .
                } WHERE {
                  ?x a nco:PersonContact ;
                      nie:generator 'test-instance-to-remove' .
                }
                """
        self.tracker.update(DELETE_SPARQL)
        count_final = self.tracker.count_instances("nco:PersonContact")
        self.assertEqual(count_before_insert, count_final)


class TrackerStorePhoneNumberTest (CommonTrackerStoreTest):
    """
    Tests around phone numbers (maemo specific). Inserting correct/incorrect ones
    and running query to get the contact from the number.
    """

    def test_phone_01(self):
        """
        1. Setting the maemo:localPhoneNumber property to last 7 digits of phone number.
        2. Receiving a message  from a contact whose localPhoneNumber is saved.
        3. Query messages from the local phone number
        """
        PhoneNumber = str(random.randint(0, sys.maxsize))
        UUID = str(time.time())
        UUID1 = str(random.randint(0, sys.maxsize))
        UUID2 = str(random.randint(0, sys.maxsize))
        localNumber = PhoneNumber[-7:]
        d = datetime.datetime.now()
        Received = d.isoformat()
        ID = int(time.time()) % 1000
        Given_Name = 'test_GN_' + repr(ID)
        Family_Name = 'test_FN_' + repr(ID)

        INSERT_CONTACT_PHONE = """
                INSERT {
                    <tel:123456789> a nco:PhoneNumber ;
                          nco:phoneNumber  '00358555444333' ;
                          maemo:localPhoneNumber '5444333'.

                    <test://test_phone_1/contact> a nco:PersonContact;
                        nco:contactUID '112';
                        nco:nameFamily 'Family-name'  ;
                        nco:nameGiven 'Given-name'.
                    <test://test_phone_1/contact> nco:hasPhoneNumber <tel:123456789>.
                }
                """
        self.tracker.update(INSERT_CONTACT_PHONE)

        INSERT_MESSAGE = """
                INSERT {
                    <test://test_phone_1/message> a nmo:Message ;
                         nmo:from [a nco:Contact ; nco:hasPhoneNumber <tel:123456789>];
                         nmo:receivedDate '2010-01-02T10:13:00Z' ;
                         nie:plainTextContent 'hello'
                }
                """
        self.tracker.update(INSERT_MESSAGE)

        QUERY_SPARQL = """
                SELECT ?msg WHERE {
                     ?msg a nmo:Message;
                         nmo:from ?c .
                     ?c nco:hasPhoneNumber ?n .
                     ?n maemo:localPhoneNumber '5444333'.
                } """
        result = self.tracker.query(QUERY_SPARQL)
        self.assertEqual(len(result), 1)
        self.assertEqual(len(result[0]), 1)
        self.assertEqual(result[0][0], "test://test_phone_1/message")

    def test_phone_02(self):
        """
        Inserting a local phone number which have spaces
        """
        INSERT_SPARQL = """
                INSERT {
                        <tel+3333333333> a nco:PhoneNumber ;
                                nco:phoneNumber  <tel+3333333333> ;
                                maemo:localPhoneNumber '333 333'.

                        <test://test_phone_02/contact> a nco:PersonContact;
                                nco:nameFamily 'test_name_01' ;
                                nco:nameGiven 'test_name_02';
                                nco:hasPhoneNumber <tel+3333333333> .
                }
                """
        self.tracker.update(INSERT_SPARQL)


if __name__ == "__main__":
    ut.main()



# Code taken and modified from unittest2 framework (case.py)

# Copyright (c) 1999-2003 Steve Purcell
# Copyright (c) 2003-2010 Python Software Foundation
# Copyright (c) 2010, Nokia (ivan.frade@nokia.com)
# Copyright (C) 2019, Sam Thursfield <sam@afuera.me.uk>

# This module is free software, and you may redistribute it and/or modify
# it under the same terms as Python itself, so long as this copyright message
# and disclaimer are retained in their original form.

# IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
# SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
# THIS CODE, EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.

# THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
# AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

"""
Write values in tracker and check the actual values are written
on the files. Note that these tests are highly platform dependant.
"""

from functools import wraps
import sys

import configuration as cfg


def expectedFailureJournal():
    """
    Decorator to handle tests that are expected to fail when journal is disabled.
    """
    def decorator(func):
        # no wrapping if journal is enabled, test is expected to pass
        if not cfg.disableJournal:
            return func

        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                func(*args, **kwargs)
            except Exception:
                raise ut.case._ExpectedFailure(sys.exc_info())
            raise Exception(
                "Unexpected success. This should fail because journal is disabled")
        return wrapper
    return decorator



#
# Copyright (C) 2010, Nokia <jean-luc.lamadon@nokia.com>
# Copyright (C) 2019, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#


import json
import logging
import os
import sys


if 'TRACKER_FUNCTIONAL_TEST_CONFIG' not in os.environ:
    raise RuntimeError("The TRACKER_FUNCTIONAL_TEST_CONFIG environment "
                       "variable must be set to point to the location of "
                       "the generated configuration.json file.")

with open(os.environ['TRACKER_FUNCTIONAL_TEST_CONFIG']) as f:
    config = json.load(f)


TOP_SRCDIR = os.path.dirname(os.path.dirname(
    os.path.dirname(os.path.dirname(os.path.dirname(__file__)))))
TOP_BUILDDIR = os.environ['TRACKER_FUNCTIONAL_TEST_BUILD_DIR']

TEST_ONTOLOGIES_DIR = config['TEST_ONTOLOGIES_DIR']
TRACKER_STORE_PATH = config['TRACKER_STORE_PATH']
disableJournal = (len(config['disableJournal']) == 0)


def get_environment_boolean(variable):
    '''Parse a yes/no boolean passed through the environment.'''

    value = os.environ.get(variable, 'no').lower()
    if value in ['no', '0', 'false']:
        return False
    elif value in ['yes', '1', 'true']:
        return True
    else:
        raise RuntimeError('Unexpected value for %s: %s' %
                           (variable, value))


if get_environment_boolean('TRACKER_TESTS_VERBOSE'):
    logging.basicConfig(stream=sys.stderr, level=logging.DEBUG)



#!/usr/bin/python3
#
# Copyright (C) 2010, Nokia <ivan.frade@nokia.com>
# Copyright (C) 2019, Sam Thursfield <sam@afuera.me.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
"""
Test that after insertion/remove/updates in the store, the signals
are emitted. Theses tests are not extensive (only few selected signals
are tested)
"""

from gi.repository import Gio
from gi.repository import GLib

import time
import unittest as ut

from storetest import CommonTrackerStoreTest as CommonTrackerStoreTest


GRAPH_UPDATED_SIGNAL = "GraphUpdated"

CONTACT_CLASS_URI = "http://www.semanticdesktop.org/ontologies/2007/03/22/nco#PersonContact"

REASONABLE_TIMEOUT = 10  # Time waiting for the signal to be emitted


class TrackerStoreSignalsTests (CommonTrackerStoreTest):
    """
    Insert/update/remove instances from nco:PersonContact
    and check that the signals are emitted
    """

    def setUp(self):
        self.clean_up_list = []

        self.loop = GLib.MainLoop()
        self.timeout_id = 0

        self.bus = Gio.bus_get_sync(Gio.BusType.SESSION, None)

        self.results_classname = None
        self.results_deletes = None
        self.results_inserts = None

    def tearDown(self):
        for uri in self.clean_up_list:
            self.tracker.update("DELETE { <%s> a rdfs:Resource }" % uri)

        self.clean_up_list = []

    def __connect_signal(self):
        """
        After connecting to the signal, call self.__wait_for_signal.
        """
        self.cb_id = self.bus.signal_subscribe(
            sender=self.tracker.TRACKER_BUSNAME,
            interface_name=self.tracker.RESOURCES_IFACE,
            member=GRAPH_UPDATED_SIGNAL,
            object_path=self.tracker.TRACKER_OBJ_PATH,
            arg0=CONTACT_CLASS_URI,
            flags=Gio.DBusSignalFlags.NONE,
            callback=self.__signal_received_cb)

    def __wait_for_signal(self):
        """
        In the callback of the signals, there should be a self.loop.quit ()
        """
        self.timeout_id = GLib.timeout_add_seconds(
            REASONABLE_TIMEOUT, self.__timeout_on_idle)
        self.loop.run()

    def __timeout_on_idle(self):
        self.loop.quit()
        self.fail("Timeout, the signal never came!")

    def __pretty_print_array(self, array):
        for g, s, o, p in array:
            uri, prop, value = self.tracker.query(
                "SELECT tracker:uri (%s), tracker:uri (%s), tracker:uri (%s) WHERE {}" % (s, o, p))
            print(" - (", "-".join([g, uri, prop, value]), ")")

    def __signal_received_cb(self, connection, sender_name, object_path, interface_name, signal_name, parameters):
        """
        Save the content of the signal and disconnect the callback
        """
        classname, deletes, inserts = parameters.unpack()

        self.results_classname = classname
        self.results_deletes = deletes
        self.results_inserts = inserts

        if (self.timeout_id != 0):
            GLib.source_remove(self.timeout_id)
            self.timeout_id = 0
        self.loop.quit()
        self.bus.signal_unsubscribe(self.cb_id)

    def test_01_insert_contact(self):
        self.clean_up_list.append("test://signals-contact-add")
        CONTACT = """
        INSERT {
        <test://signals-contact-add> a nco:PersonContact ;
             nco:nameGiven 'Contact-name added';
             nco:nameFamily 'Contact-family added';
             nie:generator 'test-14-signals' ;
             nco:contactUID '1321321312312';
             nco:hasPhoneNumber <tel:555555555> .
        }
        """
        self.__connect_signal()
        self.tracker.update(CONTACT)
        time.sleep(1)
        self.__wait_for_signal()

        # validate results
        self.assertEqual(len(self.results_deletes), 0)
        self.assertEqual(len(self.results_inserts), 6)

    def test_02_remove_contact(self):
        CONTACT = """
        INSERT {
         <test://signals-contact-remove> a nco:PersonContact ;
             nco:nameGiven 'Contact-name removed';
             nco:nameFamily 'Contact-family removed'.
        }
        """
        self.__connect_signal()
        self.tracker.update(CONTACT)
        self.__wait_for_signal()

        self.__connect_signal()
        self.tracker.update ("""
            DELETE { <test://signals-contact-remove> a rdfs:Resource }
            """)
        self.__wait_for_signal()

        # Validate results:
        self.assertEqual(len(self.results_deletes), 1)
        self.assertEqual(len(self.results_inserts), 0)

    def test_03_update_contact(self):
        self.clean_up_list.append("test://signals-contact-update")

        self.__connect_signal()
        self.tracker.update(
            "INSERT { <test://signals-contact-update> a nco:PersonContact }")
        self.__wait_for_signal()

        self.__connect_signal()
        self.tracker.update(
            "INSERT { <test://signals-contact-update> nco:fullname 'wohoo'}")
        self.__wait_for_signal()

        self.assertEqual(len(self.results_deletes), 0)
        self.assertEqual(len(self.results_inserts), 1)

    def test_04_fullupdate_contact(self):
        self.clean_up_list.append("test://signals-contact-fullupdate")

        self.__connect_signal()
        self.tracker.update(
            "INSERT { <test://signals-contact-fullupdate> a nco:PersonContact; nco:fullname 'first value' }")
        self.__wait_for_signal()

        self.__connect_signal()
        self.tracker.update ("""
               DELETE { <test://signals-contact-fullupdate> nco:fullname ?x }
               WHERE { <test://signals-contact-fullupdate> a nco:PersonContact; nco:fullname ?x }
               
               INSERT { <test://signals-contact-fullupdate> nco:fullname 'second value'}
               """)
        self.__wait_for_signal()

        self.assertEqual(len(self.results_deletes), 1)
        self.assertEqual(len(self.results_inserts), 1)


if __name__ == "__main__":
    ut.main()
